package com.btc.ep.dal.persist.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.FlushModeType;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

import com.btc.ep.dal.persist.PersistenceService2;

public class PersistenceServiceImpl2 extends PersistenceService2 {

	EntityManagerFactory entityManagerFactory;
	protected EntityManager entityManager;

	public PersistenceServiceImpl2() {
		entityManagerFactory = Persistence.createEntityManagerFactory("eppdb2");
		entityManager = entityManagerFactory.createEntityManager();
		entityManager.setFlushMode(FlushModeType.AUTO);
		EntityTransaction transaction = entityManager.getTransaction();
		transaction.begin();
	}

	protected void activate() {
	}
	
	public void open() {
		entityManager = entityManagerFactory.createEntityManager();
		entityManager.setFlushMode(FlushModeType.AUTO);
		EntityTransaction transaction = entityManager.getTransaction();
		transaction.begin();
	}

	public void close() {
		EntityTransaction transaction = entityManager.getTransaction();
		try {
			transaction.rollback();
		} finally {
			this.entityManager.close();
		}
	}

	public void clear() {
		this.entityManager.clear();
	}

	@Deprecated
	public <T> T create(T obj) {
		persist(obj);
		commit();
		return obj;
	}

	public <T> void persist(T obj) {
		EntityManager em = getEntityManager();
		em.persist(obj);
	}

	public <T> T update(T obj) {
		EntityManager em = getEntityManager();
		T aobj = em.merge(obj);
		return aobj;
	}

	public <T> void refresh(T obj) {
		EntityManager em = getEntityManager();
		em.refresh(obj);
	}

	public void commit() {
		EntityManager em = getEntityManager();
		EntityTransaction transaction = em.getTransaction();
		try {
			transaction.commit();
		} catch (Exception ex) {
			throw ex;
		} finally {
			transaction.begin();
		}
	}

	public void rollback() {
		EntityManager em = getEntityManager();
		EntityTransaction transaction = em.getTransaction();
		transaction.rollback();
		transaction.begin();
	}

	public void flush() {
		EntityManager em = getEntityManager();
		em.flush();
	}

	public EntityTransaction getTransaction() {

		EntityTransaction transaction = getEntityManager().getTransaction();

		if (transaction.isActive()) {
			return null;
		} else {
			transaction.begin();
			return transaction;
		}
	}

	public <T> void remove(Class<T> clazz, String uid) {
		EntityManager em = getEntityManager();
		T obj = em.find(clazz, uid);
		em.remove(obj);
	}

	public <T> void remove(T obj) {
		EntityManager em = getEntityManager();
		em.remove(obj);
	}

	public <T> boolean contains(T obj) {
		EntityManager em = getEntityManager();
		boolean res = em.contains(obj);
		return res;
	}

	public <T> T get(Class<T> clazz, Object uid) {
		EntityManager em = getEntityManager();
		return em.find(clazz, uid);
	}

	public <T> T find(Class<T> clazz, String uid) {

		Map<String, Object> map = new HashMap<>();
		map.put("objectdb.result-fetch", "LAZY");
		EntityManager em = getEntityManager();
		return em.find(clazz, uid, map);
	}

	public <T> T getReference(Class<T> clazz, String uid) {

		Map<String, Object> map = new HashMap<>();
		map.put("objectdb.result-fetch", "LAZY");
		EntityManager em = getEntityManager();
		return em.getReference(clazz, uid);
	}

	public <T> T getModelElementByNamedQuery(String query, Class<T> clazz, Object... args) {
		EntityManager em = getEntityManager();
		TypedQuery<T> q = em.createNamedQuery(query, clazz);
		for (int i = 0; i < args.length; i++) {
			Object arg = args[i];
			if (arg instanceof String) {
				arg = asLike((String) arg);
			}
			q.setParameter(i + 1, arg);
		}
		return q.getSingleResult();
	}

	public <T> List<T> getModelElementsByNamedQuery(String query, Class<T> clazz, Object... args) {
		EntityManager em = getEntityManager();
		TypedQuery<T> q = em.createNamedQuery(query, clazz);
		q.setFlushMode(FlushModeType.AUTO);
		for (int i = 0; i < args.length; i++) {
			Object arg = args[i];
			if (arg instanceof String) {
				arg = asLike((String) arg);
			}
			q.setParameter(i + 1, arg);
		}
		return q.getResultList();
	}

	public <T> T getModelElementByQuery(String query, Class<T> clazz, Object... args) {
		EntityManager em = getEntityManager();
		TypedQuery<T> q = em.createQuery(query, clazz);
		for (int i = 0; i < args.length; i++) {
			Object arg = args[i];
			if (arg instanceof String) {
				arg = asLike((String) arg);
			}
			q.setParameter(i + 1, arg);
		}
		return q.getSingleResult();
	}

	public <T> List<T> getModelElementsByQuery(String query, Class<T> clazz, Object... args) {
		EntityManager em = getEntityManager();
		TypedQuery<T> q = em.createQuery(query, clazz);
		q.setHint("objectdb.result-fetch", "LAZY");
		for (int i = 0; i < args.length; i++) {
			Object arg = args[i];
			if (arg instanceof String) {
				arg = asLike((String) arg);
			}
			q.setParameter(i + 1, arg);
		}
		return q.getResultList();
	}

	public <T> T getSingleResultByNamedQuery(String queryName, Class<T> clazz, Object... args) {
		EntityManager em = getEntityManager();
		TypedQuery<T> q = em.createNamedQuery(queryName, clazz);
		for (int i = 0; i < args.length; i++) {
			Object arg = args[i];
			if (arg instanceof String) {
				arg = asLike((String) arg);
			}
			q.setParameter(i + 1, arg);
		}
		return q.getSingleResult();
	}

	public <T> T getSingleResultByQuery(String query, Class<T> clazz, Object... args) {
		EntityManager em = getEntityManager();
		TypedQuery<T> q = em.createQuery(query, clazz);
		for (int i = 0; i < args.length; i++) {
			Object arg = args[i];
			if (arg instanceof String) {
				arg = asLike((String) arg);
			}
			q.setParameter(i + 1, arg);
		}
		return q.getSingleResult();
	}

	public <T> T getSpecialSingleResultByQuery(String query, Class<T> clazz, Object... args) {
		EntityManager em = getEntityManager();
		TypedQuery<T> q = em.createQuery(query, clazz);

		q.setHint("objectdb.result-fetch", "LAZY");
		for (int i = 0; i < args.length; i++) {
			Object arg = args[i];
			if (arg instanceof String) {
				arg = asLike((String) arg);
			}
			q.setParameter(i + 1, arg);
		}
		return q.getSingleResult();
	}

	public <T> List<T> getResultListByQuery(String query, Class<T> clazz, Object... args) {
		EntityManager em = getEntityManager();
		TypedQuery<T> q = em.createQuery(query, clazz);
		for (int i = 0; i < args.length; i++) {
			Object arg = args[i];
			if (arg instanceof String) {
				arg = asLike((String) arg);
			}
			q.setParameter(i + 1, arg);
		}
		return q.getResultList();
	}

	public void updateByNamedQuery(String queryName, Object... args) {

		EntityManager em = getEntityManager();
		Query query = em.createNamedQuery(queryName);
		query.executeUpdate();
	}

	public void updateQuery(String query, Object... args) {

		EntityManager em = getEntityManager();
		Query q = em.createQuery(query);
		q.executeUpdate();
	}

	private EntityManager getEntityManager() {
		return this.entityManager;
	}

	private String asLike(String name) {
		String al = name.replace('*', '%');
		return al;
	}

	public void detach(Object o) {
		getEntityManager().detach(o);
	}
}
