package optimisticlockwithremove.dmo.impl;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.persistence.Access;
import javax.persistence.AccessType;
import javax.persistence.Basic;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;

import optimisticlockwithremove.dmo.Formalization;

@Entity
@Access (AccessType.FIELD)
public class FormalizationImpl extends NamedElementImpl implements Formalization {

    @Basic
    private String scopeID;

    @ElementCollection
    private Map<String, Set<String>> executionScopeIDToObserverUids = new HashMap<>();

    protected FormalizationImpl() {
    }

    public FormalizationImpl(String scopeID, String name) {
        super(name);
        this.scopeID = scopeID;
    }

    @Override
    public String getScopeId() {
        return scopeID;
    }

    public void setScopeID(String scopeID) {
        this.scopeID = scopeID;
    }

    @Override
    public Set<String> getObserverUids(String scopeUid) {
        Set<String> observersForScope = executionScopeIDToObserverUids.get(scopeUid);
        if (observersForScope == null) {
            return Collections.emptySet();
        }
        return observersForScope;
    }

    @Override
    public void addObserverUidForScope(String scopeUid, String observerUid) {
        Set<String> scopesObserverUids = executionScopeIDToObserverUids.get(scopeUid);
        if (scopesObserverUids == null) {
            scopesObserverUids = new HashSet<>();
            executionScopeIDToObserverUids.put(scopeUid, scopesObserverUids);
        }
        scopesObserverUids.add(observerUid);
    }

    @Override
    public void cleanAllObservers() {
        executionScopeIDToObserverUids.clear();
    }

    @Override
    public Collection<String> getAllObserverUids() {
        Set<String> allObserverUids = new HashSet<>();
        for (Set<String> scopeObserverUids : executionScopeIDToObserverUids.values()) {
            allObserverUids.addAll(scopeObserverUids);
        }
        return allObserverUids;
    }
}
